/* DFBKana - a kana drill utility for DirectFB
 *
 * Copyright (C) 2001 Andreas Hundt <andi@fischlustig.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111307, USA.
 */

#include <directfb.h>
#include "config.h"

#include "dfbkana.h"
#include "infowindow.h"
#include "gridwindow.h"
#include "kanawindow.h"

/* DirectFB interfaces  */
IDirectFB               *dfb;
IDirectFBDisplayLayer   *layer;
IDirectFBEventBuffer    *keybuffer;
IDirectFBImageProvider  *provider;
IDirectFBFont           *font;
IDirectFBFont           *japanese_font = NULL;
IDirectFBSurface        *bgsurface;


int xres,yres,fontheight;

#ifdef DEFAULT_JAPANESE_FONT
char *japanese_fontname = JAPANESE_FONT;
#else
char *japanese_fontname = NULL;
#endif

Kana kanatab[50];

int sorted_kana[46];

char *katakanastring;
char *hiraganastring;

char *romanjitab[50] =
{
     "a"    , "i"    , "u"     , "e"    , "o"    ,
     "ka"   , "ki"   , "ku"    , "ke"   , "ko"   ,
     "sa"   , "shi"  , "su"    , "se"   , "so"   ,
     "ta"   , "chi"  , "tsu"   , "te"   , "to"   ,
     "na"   , "ni"   , "nu"    , "ne"   , "no"   ,
     "ha/wa", "hi"   , "hu/fu" , "he/e" , "ho"   ,
     "ma"   , "mi"   , "mu"    , "me"   , "mo"   ,
     "ya"   , "--"   , "yu"    , "--"   , "yo"   ,
     "ra/la", "ri/li", "ru/lu" , "re/le", "ro/lo",
     "wa"   , "--"   , "n/m/ng", "--"   , "wo/o"
};

/* HIRAGANA (UTF-8) */
char *hiraganatab[50] =
{
     /* a i u e o */
     "\xe3\x81\x82","\xe3\x81\x84","\xe3\x81\x86","\xe3\x81\x88","\xe3\x81\x8a",
     /* ka ki ku ke ko */
     "\xe3\x81\x8b","\xe3\x81\x8d","\xe3\x81\x8f","\xe3\x81\x91","\xe3\x81\x93",
     /* sa shi su se so */
     "\xe3\x81\x95","\xe3\x81\x97","\xe3\x81\x99","\xe3\x81\x9b","\xe3\x81\x9d",
     /* ta chi tsu te to */
     "\xe3\x81\x9f","\xe3\x81\xa1","\xe3\x81\xa4","\xe3\x81\xa6","\xe3\x81\xa8",
     /* na ni nu ne no */
     "\xe3\x81\xaa","\xe3\x81\xab","\xe3\x81\xac","\xe3\x81\xad","\xe3\x81\xae",
     /* ha hi hu he ho */
     "\xe3\x81\xaf","\xe3\x81\xb2","\xe3\x81\xb5","\xe3\x81\xb8","\xe3\x81\xbb",
     /* ma mi mu me mo */
     "\xe3\x81\xbe","\xe3\x81\xbf","\xe3\x82\x80","\xe3\x82\x81","\xe3\x82\x82",
     /* ya yu yo */
     "\xe3\x82\x84",     "--"     ,"\xe3\x82\x86",     "--"     ,"\xe3\x82\x88",
     /* ra ri ru re ro */
     "\xe3\x82\x89","\xe3\x82\x8a","\xe3\x82\x8b","\xe3\x82\x8c","\xe3\x82\x8d",
     /* wa n wo */
     "\xe3\x82\x8f",     "--"     ,"\xe3\x82\x93",     "--"     ,"\xe3\x82\x92"
};

/* KATAKANA (UTF-8) */
char *katakanatab[50] =
{
     /* a i u e o */
     "\xe3\x82\xa2","\xe3\x82\xa4","\xe3\x82\xa6","\xe3\x82\xa8","\xe3\x82\xaa",
     /* ka ki ku ke ko */
     "\xe3\x82\xab","\xe3\x82\xad","\xe3\x82\xaf","\xe3\x82\xb1","\xe3\x82\xb3",
     /* sa shi su se so */
     "\xe3\x82\xb5","\xe3\x82\xb7","\xe3\x82\xb9","\xe3\x82\xbb","\xe3\x82\xbd",
     /* ta chi tsu te to */
     "\xe3\x82\xbf","\xe3\x83\x81","\xe3\x83\x84","\xe3\x83\x86","\xe3\x83\x88",
     /* na ni nu ne no */
     "\xe3\x83\x8a","\xe3\x83\x8b","\xe3\x83\x8c","\xe3\x83\x8d","\xe3\x83\x8e",
     /* ha hi hu he ho */
     "\xe3\x83\x8f","\xe3\x83\x92","\xe3\x83\x95","\xe3\x83\x98","\xe3\x83\x9b",
     /* ma mi mu me mo */
     "\xe3\x83\x9e","\xe3\x83\x9f","\xe3\x83\xa0","\xe3\x83\xa1","\xe3\x83\xa2",
     /* ya yu yo */
     "\xe3\x83\xa4",     "--"     ,"\xe3\x83\xa6",     "--"     ,"\xe3\x83\xa8",
     /* ra ri ru re ro */
     "\xe3\x83\xa9","\xe3\x83\xaa","\xe3\x83\xab","\xe3\x83\xac","\xe3\x83\xad",
     /* wa n wo */
     "\xe3\x83\xaf",     "--"     ,"\xe3\x83\xb3",     "--"     ,"\xe3\x83\xb2"
};

void print_usage() 
{
     printf( "\n" );
     printf( "Usage:\n" );
     printf( "dfbkana --japanese-font=<filename> \n" );
     printf( "\n" );
     printf( "(hint: kochi-mincho.ttf is a good choice)\n" );
}

int parse_commandline( int argc, char* argv[] )
{
     int i;
     if (argc && argv) {
          for (i = 1; i < argc; i++) {
               if (strcmp ((argv)[i], "--help") == 0) {
                    print_usage();
                    return 1;
               }

               if (strncmp ((argv)[i], "--", 2) == 0) {
                    int len = strlen( (argv)[i] ) - 2;

                    if (len) {
                         char *name, *value;

                         name = strdup( (argv)[i] + 2 );
                         value = strchr( name, '=' );

                         if (value)
                              *value++ = '\0';

                         if (strcmp( name, "japanese-font" ) == 0) {
                              japanese_fontname = strdup( value );
                              free( name );
                              return 0;
                         }

                         free( name );
                     }
               }
          }
     }
     return 0;
}

int init_resources( int argc, char *argv[] )
{
     DFBResult err;

     srand((long)time(0));

     DFBCHECK(DirectFBInit( &argc, &argv ));

     /* create the super interface */
     DFBCHECK(DirectFBCreate( &dfb ));

     /* create an input buffer for key events */
     DFBCHECK(dfb->CreateEventBuffer( dfb, DICAPS_KEYS, &keybuffer ));

     DFBCHECK(dfb->GetDisplayLayer( dfb, DLID_PRIMARY, &layer ));

     /* load font */
     {
          DFBFontDescription desc;

          desc.flags = DFDESC_HEIGHT;
          desc.height = GRIDWINDOW_HEIGHT/20;

          DFBCHECK(dfb->CreateFont( dfb, DATADIR FONT, &desc, &font ));
          DFBCHECK(font->GetHeight( font, &fontheight ));
     }

     /* load japanese font */
     if (japanese_fontname) {
          DFBFontDescription desc;

          desc.flags = DFDESC_HEIGHT;
          desc.height = GRIDWINDOW_HEIGHT/10;

          dfb->CreateFont( dfb, japanese_fontname, &desc, &japanese_font );
     }

     if (japanese_font) {
          hiraganastring = "\xe3\x81\xb2\xe3\x82\x89\xe3\x81\x8c\xe3\x81\xaa mode";
          katakanastring = "\xe3\x82\xab\xe3\x82\xbf\xe3\x82\xab\xe3\x83\x8a mode";
     }
     else {
          printf( "could not load japanese font, please specify a valid japanese TrueType font \n" );
          print_usage();          
          return -1;
     }
          

     layer->EnableCursor ( layer, 1 );
     
     return 0;
}

void set_backgroundimage( char* filename )
{
     DFBResult                err;

     IDirectFBImageProvider  *provider;
     IDirectFBSurface        *bgsurface;
     DFBSurfaceDescription    desc;
     DFBDisplayLayerConfig    layer_config;


     layer->GetConfiguration( layer, &layer_config );

     DFBCHECK(dfb->CreateImageProvider( dfb, filename, &provider ));

     desc.flags  = DSDESC_WIDTH | DSDESC_HEIGHT;
     desc.width  = layer_config.width;
     desc.height = layer_config.height;

     xres = layer_config.width;
     yres = layer_config.height;

     DFBCHECK(dfb->CreateSurface( dfb, &desc, &bgsurface ) );
     DFBCHECK(provider->RenderTo( provider, bgsurface, NULL ));

     layer->SetBackgroundImage( layer, bgsurface );
     layer->SetBackgroundMode( layer, DLBM_IMAGE );

     provider->Release( provider );
}


void init_kanatab()
{
     int i = 0;

     for (i=0; i<50; i++) {
          kanatab[i].romanji = romanjitab[i];
          kanatab[i].hiragana_char = hiraganatab[i];
          kanatab[i].katakana_char = katakanatab[i];
          kanatab[i].rating = 0;
          kanatab[i].last_correct = 0;
     }

     /* initialize the sorted kana list, and take care of the gaps */
     for (i=0; i<36; i++)
          sorted_kana[i] = i;

     sorted_kana[36] = 37;

     for (i=37; i<44; i++)
          sorted_kana[i] = i+2;

     sorted_kana[44] = 47;
     sorted_kana[45] = 49;
}


void sort_kana()
{
     int i;
        int sort = 1;

     while(sort)
     {
          sort = 0;
          for( i=0; i<45; i++ )
          {
               if (kanatab[sorted_kana[i]].rating < kanatab[sorted_kana[i+1]].rating)
               {
                    int temp = sorted_kana[i];
                    sorted_kana[i] = sorted_kana[i+1];
                    sorted_kana[i+1] = temp;
                    sort = 1;
               }
          }
     }
}

void shuffle_kana()
{
     int i;
     for (i=0; i<1000; i++) {
          int rand1 = rand()%46;
          int rand2 = rand()%46;
          int temp = sorted_kana[rand1];
          sorted_kana[rand1] = sorted_kana[rand2];
          sorted_kana[rand2] = temp;
     }
}

void solve_kana( int kana_nr, int guessed_kana_nr, int kana_mode, int delay )
{
     if (kana_nr == guessed_kana_nr) {
          if (kanatab[kana_nr].rating < 5)
               kanatab[kana_nr].rating++;
          kanatab[kana_nr].last_correct = 1;
     }
     else {
          if (kanatab[kana_nr].rating > -5)
               kanatab[kana_nr].rating--;
          kanatab[kana_nr].last_correct = -1;      
          delay = 1; /* always wait a second if guess was wrong */      
     }
     gridwindow_redraw_cell( kana_nr, kana_mode );
     
     sort_kana();

     kanawindow_moveto( kana_nr );
     sleep( delay );
     kanawindow_moveto( -1 );
}

int random_kana()
{
     int value = rand()%1035; // 1035 = 1+2+3+...+ 45
     int i;

     for (i=0; i<46; i++) {
          value-=i;
          if (value <= 0)
               break;
     }
     return sorted_kana[i];
}


/*
 * deinitializes resources and DirectFB
 */

void deinit_resources()
{
     keybuffer->Release( keybuffer );

     if (bgsurface)
           bgsurface->Release( bgsurface );

     kanawindow_deinit();
     gridwindow_deinit();
     infowindow_deinit();

     layer->Release( layer );

     dfb->Release( dfb );
}

int main( int argc, char *argv[] )
{
     int quit = 0;
     int kana_nr = 0;
     int kana_mode = 0;
     int grid_display_mode = 0;
     int kana_window_pos = -1;
     int background_mode = 1;
     int resize = 0;
     int redraw = 0;
     int move = 0;

     if (parse_commandline( argc, argv ))
          exit(1);

     if (init_resources( argc, argv )) {
          deinit_resources();
          exit(1);
     }

     set_backgroundimage( DATADIR"DoNotEat.jpg" );

     init_kanatab();
     shuffle_kana();

     gridwindow_init( xres, yres, font, japanese_font );
     kanawindow_init( 100, 100, japanese_font );

     infowindow_init( xres, yres, japanese_font );

     gridwindow_redraw( kana_mode );

     kana_nr = random_kana();
     kanawindow_update( kana_nr, kana_mode );

     infowindow_show( kana_mode ? katakanastring : hiraganastring, 500 );


     while (!quit) {
          DFBWindowEvent evt;
          DFBInputEvent  kevt;

          while (gridwindow_getevent( &evt ) == DFB_OK) {
               switch (evt.type) {
                    case DWET_BUTTONDOWN:
                         if (evt.x > (gridwindow_getwidth() - 10)) {
                              gridwindow_grabpointer();
                              if  (evt.y > (gridwindow_getheight()- 10)) {
                                   resize = 1;
                              }
                              else {
                                   move = 1;
                              }
                         }
                         else {
                              int guessed_kana_nr = gridwindow_getkananr(evt.x, evt.y);
                                
                              solve_kana( kana_nr, guessed_kana_nr, kana_mode, 1 );
                              kana_window_pos = -1;
                              kana_nr = random_kana();
                              kanawindow_update( kana_nr, kana_mode );
                        }
                         break;                                            
                    case DWET_WHEEL:
                         gridwindow_changeopacity( evt.step );
                         break;
                    case DWET_SIZE:
                         gridwindow_setsize( evt.w - 10, evt.h -10 );
                         kanawindow_setsize( gridwindow_getwidth()/5.0f +1,
                                             gridwindow_getheight()/ 10.0f +1);


                         redraw = 1;
                         break;
                    case DWET_MOTION:
                         if (resize) {
                              gridwindow_setsize( evt.x, evt.y );
                              kanawindow_setsize( gridwindow_getwidth()/5.0f +1,
                                                  gridwindow_getheight()/ 10.0f +1);
                              redraw = 1;
                         }
                         else if (move) {
                              gridwindow_setposition( evt.cx - gridwindow_getwidth(), evt.cy -gridwindow_getheight()/2 );
                         }                         
                         break;
                    case DWET_BUTTONUP:
                         resize = 0;
                         move = 0;
                         gridwindow_ungrabpointer();
                         break;
                    default:
                        break;
               }
          }
          if (redraw) {
               gridwindow_resize();
               kanawindow_resize();           

               gridwindow_redraw( kana_mode );
               kanawindow_update( kana_nr, kana_mode );
               redraw = 0;
          }

          while (kanawindow_getevent( &evt ) == DFB_OK) {
               if (evt.type == DWET_BUTTONDOWN) {
                    kana_mode= !kana_mode;
                    if (grid_display_mode)
                         gridwindow_redraw( kana_mode );     

                    kanawindow_update( kana_nr, kana_mode );

                    infowindow_show( kana_mode ?
                                      katakanastring : hiraganastring, 500 );
               }
          }

          while (keybuffer->GetEvent( keybuffer, DFB_EVENT(&kevt) ) == DFB_OK) {
               if (kevt.type == DIET_KEYPRESS)
                    switch (kevt.keycode) {
                    case DIKC_LEFT:
                         if (--kana_window_pos < -1)
                              kana_window_pos = -1;
                         kanawindow_moveto( kana_window_pos );
                         break;
                    case DIKC_RIGHT:
                         if (++kana_window_pos > 49)
                              kana_window_pos = 49;
                         kanawindow_moveto( kana_window_pos );
                         break;
                    case DIKC_UP:
                         kana_window_pos-=5;
                         if (kana_window_pos < -1)
                              kana_window_pos = -1;
                         kanawindow_moveto( kana_window_pos );
                         break;
                    case DIKC_DOWN:
                         kana_window_pos+=5;
                         if (kana_window_pos > 49)
                              kana_window_pos = 49;
                         kanawindow_moveto( kana_window_pos );
                         break;
                    case DIKC_ENTER:
                    case DIKC_OK:
                         if (kana_window_pos != -1) {
                              solve_kana( kana_nr, kana_window_pos, kana_mode, 0 );
                              kana_window_pos = -1;
                              kana_nr = random_kana();
                              kanawindow_update( kana_nr, kana_mode );
                         }
                         break;
                    case DIKC_R:
                    case DIKC_2:                    
                         grid_display_mode = !grid_display_mode;
                         gridwindow_setdisplaymode( grid_display_mode );
                         gridwindow_redraw( kana_mode );
                         infowindow_show( grid_display_mode ?
                                          "romanji off" : "romanji on",
                                          500 );                         
                         break;
                    case DIKC_1:
                    case DIKC_Q:
                         kana_mode = !kana_mode;
                         if (grid_display_mode)
                              gridwindow_redraw( kana_mode );
                         kanawindow_update( kana_nr, kana_mode );
                         infowindow_show( kana_mode ?
                                          katakanastring : hiraganastring,
                                          500 );
                         break;
                    case DIKC_B:
                    case DIKC_3:
                         background_mode = !background_mode;
                         if (background_mode)
                              layer->SetBackgroundMode( layer, DLBM_IMAGE );
                         else
                              layer->SetBackgroundMode( layer, DLBM_COLOR );
                         break;
                    case DIKC_ESCAPE:
                         quit =1;
                    default:
                         break;
               }
          }
          usleep( 20000 );
     }

     deinit_resources();
     return 42;
}

