/* DFBKana - a kana drill utility for DirectFB
 *
 * Copyright (C) 2001 Andreas Hundt <andi@fischlustig.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <directfb.h>
#include "config.h"

#include "dfbkana.h"
#include "gridwindow.h"

/* DirectFB interfaces  */
IDirectFBWindow         *gridwindow = NULL;
IDirectFBEventBuffer    *gridwindowbuffer = NULL;
IDirectFBFont           *gridwindowfont;
IDirectFBFont           *gridwindowfont_japanese;

static int               displaymode = 0;
static int               width  = GRIDWINDOW_WIDTH;
static int               height = GRIDWINDOW_HEIGHT;

void gridwindow_init( int x, int y, IDirectFBFont *font, IDirectFBFont *japanese_font )
{
     DFBResult err;

     DFBWindowDescription desc;

     desc.flags = ( DWDESC_POSX | DWDESC_POSY | DWDESC_WIDTH | DWDESC_HEIGHT );

     desc.width  = GRIDWINDOW_WIDTH  + 10;
     desc.height = GRIDWINDOW_HEIGHT + 10;
     desc.posx   = (x - desc.width ) / 2;
     desc.posy   = (y - desc.height) / 2;

     DFBCHECK(layer->CreateWindow( layer, &desc, &gridwindow ) );

     gridwindowfont = font;
     gridwindowfont_japanese = japanese_font;

     gridwindow->SetOpacity( gridwindow, 0xB0 );

     gridwindow->CreateEventBuffer( gridwindow, &gridwindowbuffer );
}

void gridwindow_setdisplaymode( int _displaymode )
{
     displaymode = _displaymode;
}
void gridwindow_redraw( int kana_mode )
{
     int x;
     int y;
     int i;
     float xstep = (float)(width ) /  5.0f;
     float ystep = (float)(height) / 10.0f;

     IDirectFBSurface *gridsurface;

     gridwindow->GetSurface( gridwindow, &gridsurface );

     /*clear*/
     gridsurface->SetColor( gridsurface, 0x00, 0x00, 0x00, 0xFF );
     gridsurface->FillRectangle( gridsurface, 0, height, width + 10, 10 );
     gridsurface->FillRectangle( gridsurface, width, 0, 10, height + 10 );

     gridsurface->SetColor( gridsurface, 0x00, 0x00, 0xF0, 0xFF );

     i = 6;
     while(i--) {
          x = i * xstep;
          gridsurface->DrawLine( gridsurface, x, 0, x, height );
     }

     i = 11;
     while (i--) {
          y = i * ystep;
          gridsurface->DrawLine( gridsurface, 0 , y, width, y );
     }


     for( i = 0; i<50; i++ )
          gridwindow_redraw_cell( i, kana_mode );
       
     gridsurface->Flip( gridsurface, NULL, 0 );
     gridsurface->Release( gridsurface );
}


void gridwindow_redraw_cell( int kana_nr, int kana_mode )
{
     int fontheight;
 
     int kana_x = kana_nr%5;
     int kana_y = kana_nr/5;           
     float xstep = (float)(width ) /  5.0f;
     float ystep = (float)(height) / 10.0f;
     int x = kana_x * xstep +1;
     int y = kana_y * ystep +1;
     int h = (int)((kana_y + 1) * ystep) - y;
     int w = (int)((kana_x + 1) * xstep) - x;

     IDirectFBSurface *gridsurface;

     gridwindow->GetSurface( gridwindow, &gridsurface );

     
     if (kanatab[kana_nr].rating < 0)
          gridsurface->SetColor( gridsurface, -kanatab[kana_nr].rating/6.0f * 255, 0x00, 0x00, 0xFF );
     else if (kanatab[kana_nr].rating > 0)
          gridsurface->SetColor( gridsurface, 0x00, kanatab[kana_nr].rating/6.0f * 255, 0x00, 0xFF );
     else 
          gridsurface->SetColor( gridsurface, 0x00, 0x00, 0x00, 0xFF );
     /* calculated above */
     gridsurface->FillRectangle( gridsurface, x , y, w, h );

     switch (kanatab[kana_nr].last_correct) {
          case -1: gridsurface->SetColor( gridsurface, 0xFF, 0x00, 0x00, 0xFF );
                   break;
          case 1:  gridsurface->SetColor( gridsurface, 0x00, 0xFF, 0x00, 0xFF );
                   break;
          default: gridsurface->SetColor( gridsurface, 0xFF, 0xFF, 0xFF, 0xFF );
                   break;
     }

     if (displaymode == 0) {
          gridwindowfont->GetHeight( gridwindowfont, &fontheight );
          gridsurface->SetFont( gridsurface, gridwindowfont );
          gridsurface->DrawString( gridsurface,  kanatab[kana_nr].romanji,
                                   -1, kana_x*xstep + xstep/2,
                                       kana_y*ystep + ystep/2 - fontheight/2,
                                       DSTF_CENTER | DSTF_TOP );
     }
     else {
          gridwindowfont_japanese->GetHeight( gridwindowfont_japanese, &fontheight );
          gridsurface->SetFont( gridsurface, gridwindowfont_japanese);
          if (kana_mode == 0) {
               gridsurface->DrawString( gridsurface,  kanatab[kana_nr].katakana_char,
                                        -1, kana_x*xstep + xstep/2,
                                            kana_y*ystep + ystep/2 - fontheight/2,
                                            DSTF_CENTER | DSTF_TOP );
          }
          else {
               gridsurface->DrawString( gridsurface,  kanatab[kana_nr].hiragana_char,
                                        -1, kana_x*xstep + xstep/2,
                                            kana_y*ystep + ystep/2 - fontheight/2,
                                            DSTF_CENTER | DSTF_TOP );
          }     
     }
     gridsurface->Release( gridsurface );
}

int gridwindow_getkananr( int x, int y )
{
     float xstep = (float)(width ) /  5.0f;
     float ystep = (float)(height) / 10.0f;
     
     return (int)(y/ystep)*5 +(int)(x/xstep);  
}
 

DFBResult gridwindow_setposition( int x, int y )
{
     return gridwindow->MoveTo( gridwindow, x, y );
}

DFBResult gridwindow_getposition( int *x, int *y )
{
     return gridwindow->GetPosition( gridwindow, x, y );
}

void gridwindow_changeopacity( int step )
{
     unsigned char opacity;
     int tmp_opacity;

     gridwindow->GetOpacity( gridwindow, &opacity );
     tmp_opacity = opacity + step * 4;
     if (tmp_opacity > 255)
          tmp_opacity = 255;
     if (tmp_opacity <= 0)
          tmp_opacity = 1;

     opacity = tmp_opacity;

     gridwindow->SetOpacity( gridwindow, opacity );
}

void gridwindow_setsize( int w, int h )
{
     if (w < 20)
          w = 20;
     
     if (h < 20)
          h = 20;
     width  = w;
     height = h;
}

DFBResult gridwindow_resize()
{
     return gridwindow->Resize( gridwindow, width + 10, height + 10 );
}

int gridwindow_getwidth()
{
     return width;
}

int gridwindow_getheight()
{
     return height;
}

DFBResult gridwindow_grabpointer()
{
     return gridwindow->GrabPointer( gridwindow );
}

DFBResult gridwindow_ungrabpointer()
{
     return gridwindow->UngrabPointer( gridwindow );
}

DFBResult gridwindow_getevent( DFBWindowEvent *wevt)
{
     return gridwindowbuffer->GetEvent( gridwindowbuffer, DFB_EVENT(wevt) );
}

void gridwindow_deinit()
{
     if (gridwindowbuffer) {          
          gridwindowbuffer->Release( gridwindowbuffer );
          gridwindowbuffer = NULL;
     }
     if (gridwindow) {
          gridwindow->Release( gridwindow );
          gridwindow = NULL;
     }
}
