/* DFBKana - a kana drill utility for DirectFB
 *
 * Copyright (C) 2001 Andreas Hundt <andi@fischlustig.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
                   
#include "dfbkana.h"
#include "infowindow.h"

/* DirectFB interfaces  */
IDirectFBWindow         *infowindow = NULL;
IDirectFBFont           *infowindowfont;

void infowindow_init( int x, int y, IDirectFBFont *font )
{
     DFBResult err;
          
     DFBWindowDescription desc;
     IDirectFBSurface *infosurface;

     desc.flags = ( DWDESC_POSX | DWDESC_POSY | DWDESC_WIDTH | DWDESC_HEIGHT );
     
     desc.posx   = (x - INFOWINDOW_WIDTH ) / 2;
     desc.posy   = (y - INFOWINDOW_HEIGHT) / 2;
     desc.width  = INFOWINDOW_WIDTH;
     desc.height = INFOWINDOW_HEIGHT;

     DFBCHECK(layer->CreateWindow( layer, &desc, &infowindow ) );

     infowindow->GetSurface( infowindow, &infosurface );
     infosurface->SetColor( infosurface, INFOWINDOW_COLOR , 0xFF );
     
     infosurface->FillRectangle( infosurface, 0, 0, INFOWINDOW_WIDTH, INFOWINDOW_BORDER );
     infosurface->FillRectangle( infosurface, INFOWINDOW_WIDTH - INFOWINDOW_BORDER, 0, INFOWINDOW_BORDER, INFOWINDOW_HEIGHT );
     infosurface->FillRectangle( infosurface, 0, INFOWINDOW_HEIGHT - INFOWINDOW_BORDER , INFOWINDOW_WIDTH, INFOWINDOW_BORDER );
     infosurface->FillRectangle( infosurface, 0, 0, INFOWINDOW_BORDER, INFOWINDOW_HEIGHT );
     
     infowindowfont = font;
}

void infowindow_show( char *text, int duration )
{   
     IDirectFBSurface *infosurface;
     int fontheight;
     
     unsigned long time, interval;

     infowindow->GetSurface( infowindow, &infosurface );

     infosurface->SetColor( infosurface, 0x00, 0x00, 0x00, 0xFF );
     infosurface->FillRectangle( infosurface, INFOWINDOW_BORDER,
                                              INFOWINDOW_BORDER,
                                              INFOWINDOW_WIDTH  - INFOWINDOW_BORDER*2,
                                              INFOWINDOW_HEIGHT - INFOWINDOW_BORDER*2 );

     infosurface->SetColor ( infosurface, 0xFF, 0xFF, 0xFF, 0xFF );
        
     infosurface->SetFont( infosurface, infowindowfont );
     infowindowfont->GetHeight( infowindowfont, &fontheight );     
             
     infosurface->DrawString( infosurface, text, -1, 
                                              INFOWINDOW_WIDTH/2,
                                             (INFOWINDOW_HEIGHT-fontheight)/2,
                                              DSTF_CENTER | DSTF_TOP );

     infosurface->Release( infosurface );
        
     time = myclock();                
     do {
          interval = myclock() - time;
             
          infowindow->SetOpacity( infowindow, (__u8)((interval/500.0f)*0xB0));
           
     } while (interval < 500);
        

     usleep ( duration * 1000 );
                
     time = myclock();        
     do {
          interval = myclock() - time;
          if (interval <= 500)     
               infowindow->SetOpacity( infowindow, (__u8)(((500-interval)/500.0f)*0xB0));

     } while (interval < 500);
        
     infowindow->SetOpacity( infowindow, 0x00 );                        
}

void infowindow_deinit()
{
     if (infowindow)
          infowindow->Release( infowindow );
}