/* DFBKana - a kana drill utility for DirectFB
 *
 * Copyright (C) 2001 Andreas Hundt <andi@fischlustig.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <directfb.h>
#include "config.h"

#include "dfbkana.h"
#include "kanawindow.h"
#include "gridwindow.h"

/* DirectFB interfaces  */
IDirectFBWindow         *kanawindow = NULL;
IDirectFBEventBuffer    *kanawindowbuffer = NULL;
IDirectFBFont           *kanawindowfont = NULL;

static int width  = GRIDWINDOW_WIDTH  / 5;
static int height = GRIDWINDOW_HEIGHT / 10;

void kanawindow_init( int x, int y, IDirectFBFont *font )
{
     DFBResult err;

     DFBWindowDescription desc;

     desc.flags = ( DWDESC_POSX | DWDESC_POSY | DWDESC_WIDTH | DWDESC_HEIGHT );

     desc.posx   = x;
     desc.posy   = y;
     desc.width  = GRIDWINDOW_WIDTH  / 5;
     desc.height = GRIDWINDOW_HEIGHT / 10;

     DFBCHECK(layer->CreateWindow( layer, &desc, &kanawindow ) );

     kanawindowfont = font;

     kanawindow->SetOpacity( kanawindow, 0xB0 );

     kanawindow->CreateEventBuffer( kanawindow, &kanawindowbuffer );
}

int kanawindow_update( int kana_nr, int kanamode )
{
     IDirectFBSurface *kanawindow_surface;
     int fontheight;

     kanawindow->GetSurface( kanawindow, &kanawindow_surface );

     kanawindow_surface->SetColor( kanawindow_surface, 0xFF, 0xFF, 0xFF, 0xFF );
     kanawindow_surface->FillRectangle( kanawindow_surface,0,0, width, height );


     kanawindow_surface->SetColor( kanawindow_surface,
                                        0x00, 0x00, 0x00, 0xFF );

     kanawindow_surface->SetFont( kanawindow_surface, kanawindowfont );

     kanawindowfont->GetHeight( kanawindowfont, &fontheight );
     if (kanamode == 0)  {
          kanawindow_surface->DrawString( kanawindow_surface,
                                          kanatab[kana_nr].hiragana_char,
                                          -1,
                                          width/2,
                                          (height-fontheight)/2,
                                          DSTF_CENTER | DSTF_TOP);
     }
     else {
          kanawindow_surface->DrawString( kanawindow_surface,
                                          kanatab[kana_nr].katakana_char,
                                          -1,
                                          width/2,
                                          (height-fontheight)/2,
                                          DSTF_CENTER | DSTF_TOP);
     }

     kanawindow_surface->Flip( kanawindow_surface, NULL, 0 );
     kanawindow_surface->Release( kanawindow_surface );

     return kana_nr;
}



void kanawindow_moveto( int kana_nr )
{
     int grid_x = 0;
     int grid_y = 0;

     if ((kana_nr < -1) || (kana_nr > 49))
          return;

     if (kana_nr == -1) {
         kanawindow->MoveTo( kanawindow, 100, 100 );
     }
     else {
          float xstep = (float)(gridwindow_getwidth())  /  5.0f;
          float ystep = (float)(gridwindow_getheight()) / 10.0f;

          gridwindow_getposition( &grid_x, &grid_y );

          kanawindow->MoveTo( kanawindow, grid_x+1 + (kana_nr%5) * xstep,
                                          grid_y+1 + (kana_nr/5) * ystep );
     }
}

DFBResult kanawindow_resize()
{
     return kanawindow->Resize( kanawindow, width, height );
}

void kanawindow_setsize( int w, int h )
{
     height = h;
     width = w;
}

DFBResult kanawindow_getevent( DFBWindowEvent *wevt)
{
     return kanawindowbuffer->GetEvent( kanawindowbuffer, DFB_EVENT(wevt) );
}

void kanawindow_deinit()
{
     if (kanawindowbuffer) {
          kanawindowbuffer->Release( kanawindowbuffer );
          kanawindowbuffer = NULL;
     }
     if (kanawindow) {
          kanawindow->Release( kanawindow );
          kanawindow = NULL;
     }
}